//////////////////////////////////////////////////////////////////////////////
// Name:        SVGCanvasPathLibart.cpp
// Purpose:     Libart canvas path
// Author:      Alex Thuering
// Created:     2005/05/06
// RCS-ID:      $Id: SVGCanvasPathLibart.cpp,v 1.2 2005/05/13 20:14:11 ntalex Exp $
// Copyright:   (c) 2005 Alex Thuering
// Licence:     wxWindows licence
//////////////////////////////////////////////////////////////////////////////

#include "SVGCanvasPathLibart.h"
#include <wx/log.h>

#include <libart_lgpl/libart.h>

wxSVGCanvasPathLibart::wxSVGCanvasPathLibart()
{
  m_count = 0;
  m_size = 16;
  m_moveto_idx = -1;
  m_bpath = art_new(ArtBpath, m_size);
}

wxSVGCanvasPathLibart::~wxSVGCanvasPathLibart()
{
  art_free(m_bpath);
}

ArtBpath& wxSVGCanvasPathLibart::AddBPath()
{
  if (m_count == m_size)
  {
	m_size *= 2;
	ArtBpath* bpath = art_new(ArtBpath, m_size);
	memcpy(bpath, m_bpath, m_count*sizeof(ArtBpath)); 
	art_free(m_bpath);
    m_bpath = bpath;
  }
  m_count++;
  return m_bpath[m_count-1];
}

void wxSVGCanvasPathLibart::End()
{
  ArtBpath& bpath = AddBPath();
  bpath.code = ART_END;
}

wxSVGRect wxSVGCanvasPathLibart::GetBBox()
{
  ArtDRect rect;
  ArtVpath* vec = art_bez_path_to_vec(m_bpath, 0.25);
  art_vpath_bbox_drect(vec, &rect);
  art_free(vec);
  return wxSVGRect(rect.x0, rect.y0, rect.x1-rect.x0, rect.y1-rect.y0);
}

void wxSVGCanvasPathLibart::MoveToImpl(double x, double y)
{
  ArtBpath& bpath = AddBPath();
  bpath.code = ART_MOVETO_OPEN;
  bpath.x3 = x;
  bpath.y3 = y;
  m_moveto_idx = m_count - 1;
}

void wxSVGCanvasPathLibart::LineToImpl(double x, double y)
{
  ArtBpath& bpath = AddBPath();
  bpath.code = ART_LINETO;
  bpath.x3 = x;
  bpath.y3 = y;
}

void wxSVGCanvasPathLibart::CurveToCubicImpl(double x1, double y1,
 double x2, double y2, double x, double y)
{
  ArtBpath& bpath = AddBPath();
  bpath.code = ART_CURVETO;
  bpath.x1 = x1;
  bpath.y1 = y1;
  bpath.x2 = x2;
  bpath.y2 = y2;
  bpath.x3 = x;
  bpath.y3 = y;
}

bool wxSVGCanvasPathLibart::ClosePathImpl()
{
  if (!m_count || m_moveto_idx == -1)
	return false;
  
  bool end = false;
  if (m_bpath[m_count-1].code == ART_END)
  {
     m_count--;
     end = true;
     if (!m_count)
       return false;
  }
  
  if (m_bpath[m_moveto_idx].x3 != m_bpath[m_count-1].x3 ||
	  m_bpath[m_moveto_idx].y3 != m_bpath[m_count-1].y3)
	LineToImpl(m_bpath[m_moveto_idx].x3, m_bpath[m_moveto_idx].y3);
  m_bpath[m_moveto_idx].code = ART_MOVETO;
  
  if (end)
    End();
  
  return true;
}
